// eslint-disable-next-line import/no-extraneous-dependencies
import { kebabCase } from 'lodash';
// eslint-disable-next-line import/no-extraneous-dependencies
import { rgba } from 'polished';
// eslint-disable-next-line import/no-extraneous-dependencies
import React, { FC, useState } from 'react';
// eslint-disable-next-line import/no-extraneous-dependencies
import styled from 'styled-components';

// eslint-disable-next-line import/no-extraneous-dependencies
import { globalTheme, useMixin } from '@clds/component-theme';
// eslint-disable-next-line import/no-extraneous-dependencies
import { InlineFieldLayout, Label } from '@clds/dev-components';
// eslint-disable-next-line @nrwl/nx/enforce-module-boundaries,import/no-extraneous-dependencies
import Toggle from '@clds/toggle';
// eslint-disable-next-line @nrwl/nx/enforce-module-boundaries,import/no-extraneous-dependencies
import Tooltip from '@clds/tooltip';
import type { WebsiteKind, WebsiteSize, WebsiteType } from '@clds/typography';
// eslint-disable-next-line import/no-extraneous-dependencies,no-duplicate-imports
import { websiteTheme, WebsiteTypography } from '@clds/typography';

const isContrast = (type: WebsiteType) =>
  type.toLowerCase().includes('contrast');

export interface DemoProps {
  kind: WebsiteKind;
  size: WebsiteSize;
  type: WebsiteType;
  caption: string;
}

const DemoRoot = styled.div<{ invert: boolean }>`
  padding: ${globalTheme.spacing.md};
  background: ${({ invert }) =>
    invert ? globalTheme.palette.brand.darkBlue : globalTheme.palette.surface};
  position: relative;

  ${WebsiteTypography} {
    margin-bottom: ${globalTheme.spacing.md};
    display: flex;
    span {
      flex: 1;
    }
  }
`;

const CaptionText = styled.span<{ invert?: boolean }>`
  font-family: Inter, serif;
  font-size: 18px;
  color: ${(props) => rgba(props.invert ? '#ffffff' : '#000000', 0.5)};
`;

const SpecText = styled(Label)<{ invert?: boolean }>`
  font-family: Inter, serif;
  font-size: 14px;
  color: ${(props) => rgba(props.invert ? '#ffffff' : '#000000', 0.7)};
`;

const CheckboxWrapper = styled.div`
  display: flex;
  position: absolute;
  top: ${globalTheme.spacing.md};
  right: ${globalTheme.spacing.md};
  > :first-child {
    margin-right: ${globalTheme.spacing.md};
  }
`;

export const Demo: FC<DemoProps> = ({ kind, size, type, caption }) => {
  const theme = useMixin(websiteTheme);
  const [codeVisible, setCodeVisible] = useState(false);

  const kindBranch = theme[kind];
  const sizeBranch =
    kindBranch[size as Extract<keyof typeof kindBranch, WebsiteSize>];
  const typeBranch =
    sizeBranch[type as Extract<keyof typeof sizeBranch, WebsiteType>];

  const fontFamily = kindBranch.fontFamily;
  const fontWeight = typeBranch.fontWeight;
  const fontSize = sizeBranch.fontSize;
  const lineHeight = sizeBranch.lineHeight;

  const invert = isContrast(type);

  return (
    <DemoRoot invert={invert}>
      <CaptionText invert={invert}>{caption}</CaptionText>
      <Tooltip
        title={
          <>
            website/{kind}/{size}/{kebabCase(type)}
          </>
        }
      >
        <WebsiteTypography {...{ kind, size, type }}>
          <Tooltip.OverflowTrigger>
            website/{kind}/{size}/{kebabCase(type)}
          </Tooltip.OverflowTrigger>
        </WebsiteTypography>
      </Tooltip>
      <SpecText invert={invert}>
        {fontFamily}, {fontWeight}, {fontSize}, Line height: {lineHeight}
      </SpecText>
      <CheckboxWrapper>
        <InlineFieldLayout>
          <Toggle
            id={`toggle-${kind}-${type}-${size}`}
            value={codeVisible}
            onChange={setCodeVisible}
          />
          <SpecText htmlFor={`toggle-${kind}-${type}-${size}`} invert={invert}>
            Show code
          </SpecText>
        </InlineFieldLayout>
      </CheckboxWrapper>
      {codeVisible && (
        <SpecText as="code" invert={invert}>
          <pre>
            {`import { website, WebsiteTypography } from '@clds/typography';

const MixinTreeUsage = styled.div\`
  \${website.${kind}.${size}.${type}};
\`;

const ComponentUsage = () => (
    <WebsiteTypography kind="${kind}" size="${size}" type="${type}">
        your text here
    </WebsiteTypography>
);
`}
          </pre>
        </SpecText>
      )}
    </DemoRoot>
  );
};
